@@ -5,7 +5,7 @@ import click
 from ..decorators import loses_interactivity, require_bins, require_login
 from ..helpers.ansible import run_ansible
 from ..helpers.global_options import GlobalOptions
-from ..helpers.options import aws_profile_options, resource_argument
+from ..helpers.options import ansible_options, resource_argument
 from .sym import sym
 
 
@@ -15,7 +15,7 @@ from .sym import sym
 )
 @resource_argument
 @click.argument("command", nargs=-1)
-@aws_profile_options
+@ansible_options
 @click.make_pass_decorator(GlobalOptions)
 @loses_interactivity
 @require_bins("ansible-playbook", "aws", "session-manager-plugin")
@@ -26,6 +26,9 @@ def ansible_playbook(
     command: Tuple[str, ...],
     ansible_aws_profile: Optional[str],
     ansible_sym_resource: Optional[str],
+    control_master: bool,
+    send_command: bool,
+    forks: int,
 ) -> None:
     """Use approved creds for RESOURCE to run an Ansible command."""
     client = options.create_saml_client(resource)
@@ -35,4 +38,7 @@ def ansible_playbook(
         binary="ansible-playbook",
         ansible_aws_profile=ansible_aws_profile,
         ansible_sym_resource=ansible_sym_resource,
+        control_master=control_master,
+        send_command=send_command,
+        forks=forks,
     )
